@@ -1,9 +1,10 @@
+#!/usr/bin/env python3
 import asyncio
 
 import aiohttp
 
 
-async def fetch(session):
+async def fetch(session: aiohttp.ClientSession) -> None:
     print("Query http://httpbin.org/get")
     async with session.get("http://httpbin.org/get") as resp:
         print(resp.status)
