@@ -3,9 +3,11 @@ import contextlib
 import gc
 import io
 import json
+import sys
+import warnings
 from collections import deque
-from http.cookies import SimpleCookie
-from typing import Any, Awaitable, Callable, List
+from http.cookies import BaseCookie, SimpleCookie
+from typing import Any, Awaitable, Callable, Iterator, List, Optional, cast
 from unittest import mock
 from uuid import uuid4
 
@@ -15,7 +17,7 @@ from re_assert import Matches
 from yarl import URL
 
 import aiohttp
-from aiohttp import CookieJar, client, hdrs, web
+from aiohttp import CookieJar, abc, client, hdrs, web
 from aiohttp.client import ClientSession
 from aiohttp.client_proto import ResponseHandler
 from aiohttp.client_reqrep import ClientRequest
@@ -310,32 +312,91 @@ async def test_create_connector(create_session, loop, mocker) -> None:
     assert connector.close.called
 
 
+@pytest.mark.skipif(
+    sys.version_info < (3, 11),
+    reason="Use test_ssl_shutdown_timeout_passed_to_connector_pre_311 for Python < 3.11",
+)
 async def test_ssl_shutdown_timeout_passed_to_connector() -> None:
-    # Test default value
+    # Test default value (no warning expected)
     async with ClientSession() as session:
         assert isinstance(session.connector, TCPConnector)
-        assert session.connector._ssl_shutdown_timeout == 0.1
+        assert session.connector._ssl_shutdown_timeout == 0
 
-    # Test custom value
-    async with ClientSession(ssl_shutdown_timeout=1.0) as session:
-        assert isinstance(session.connector, TCPConnector)
-        assert session.connector._ssl_shutdown_timeout == 1.0
+    # Test custom value - expect deprecation warning
+    with pytest.warns(
+        DeprecationWarning, match="ssl_shutdown_timeout parameter is deprecated"
+    ):
+        async with ClientSession(ssl_shutdown_timeout=1.0) as session:
+            assert isinstance(session.connector, TCPConnector)
+            assert session.connector._ssl_shutdown_timeout == 1.0
 
-    # Test None value
-    async with ClientSession(ssl_shutdown_timeout=None) as session:
-        assert isinstance(session.connector, TCPConnector)
-        assert session.connector._ssl_shutdown_timeout is None
+    # Test None value - expect deprecation warning
+    with pytest.warns(
+        DeprecationWarning, match="ssl_shutdown_timeout parameter is deprecated"
+    ):
+        async with ClientSession(ssl_shutdown_timeout=None) as session:
+            assert isinstance(session.connector, TCPConnector)
+            assert session.connector._ssl_shutdown_timeout is None
 
     # Test that it doesn't affect when custom connector is provided
-    custom_conn = TCPConnector(ssl_shutdown_timeout=2.0)
-    async with ClientSession(
-        connector=custom_conn, ssl_shutdown_timeout=1.0
-    ) as session:
-        assert session.connector is not None
-        assert isinstance(session.connector, TCPConnector)
-        assert (
-            session.connector._ssl_shutdown_timeout == 2.0
-        )  # Should use connector's value
+    with pytest.warns(
+        DeprecationWarning, match="ssl_shutdown_timeout parameter is deprecated"
+    ):
+        custom_conn = TCPConnector(ssl_shutdown_timeout=2.0)
+    with pytest.warns(
+        DeprecationWarning, match="ssl_shutdown_timeout parameter is deprecated"
+    ):
+        async with ClientSession(
+            connector=custom_conn, ssl_shutdown_timeout=1.0
+        ) as session:
+            assert session.connector is not None
+            assert isinstance(session.connector, TCPConnector)
+            assert (
+                session.connector._ssl_shutdown_timeout == 2.0
+            )  # Should use connector's value
+
+
+@pytest.mark.skipif(
+    sys.version_info >= (3, 11),
+    reason="This test is for Python < 3.11 runtime warning behavior",
+)
+async def test_ssl_shutdown_timeout_passed_to_connector_pre_311() -> None:
+    """Test that both deprecation and runtime warnings are issued on Python < 3.11."""
+    # Test custom value - expect both deprecation and runtime warnings
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        async with ClientSession(ssl_shutdown_timeout=1.0) as session:
+            assert isinstance(session.connector, TCPConnector)
+            assert session.connector._ssl_shutdown_timeout == 1.0
+        # Should have deprecation warnings (from ClientSession and TCPConnector) and runtime warning
+        # ClientSession emits 1 DeprecationWarning, TCPConnector emits 1 DeprecationWarning + 1 RuntimeWarning = 3 total
+        assert len(w) == 3
+        deprecation_count = sum(
+            1 for warn in w if issubclass(warn.category, DeprecationWarning)
+        )
+        runtime_count = sum(
+            1 for warn in w if issubclass(warn.category, RuntimeWarning)
+        )
+        assert deprecation_count == 2  # One from ClientSession, one from TCPConnector
+        assert runtime_count == 1  # One from TCPConnector
+
+    # Test with custom connector
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        custom_conn = TCPConnector(ssl_shutdown_timeout=2.0)
+        # Should have both deprecation and runtime warnings
+        assert len(w) == 2
+    with pytest.warns(
+        DeprecationWarning, match="ssl_shutdown_timeout parameter is deprecated"
+    ):
+        async with ClientSession(
+            connector=custom_conn, ssl_shutdown_timeout=1.0
+        ) as session:
+            assert session.connector is not None
+            assert isinstance(session.connector, TCPConnector)
+            assert (
+                session.connector._ssl_shutdown_timeout == 2.0
+            )  # Should use connector's value
 
 
 def test_connector_loop(loop: asyncio.AbstractEventLoop) -> None:
@@ -361,6 +422,26 @@ def test_connector_loop(loop: asyncio.AbstractEventLoop) -> None:
         another_loop.run_until_complete(connector.close())
 
 
+def test_auto_created_connector_uses_session_loop(
+    loop: asyncio.AbstractEventLoop,
+) -> None:
+    """Test that auto-created TCPConnector uses the session's loop."""
+    # Create a ClientSession without providing a connector
+    # The session should auto-create a TCPConnector with the provided loop
+    session = ClientSession(loop=loop)
+
+    # Verify the connector was created
+    assert session.connector is not None
+    assert isinstance(session.connector, TCPConnector)
+
+    # Verify the connector uses the same loop as the session
+    assert session.connector._loop is loop
+    assert session.connector._loop is session._loop
+
+    # Clean up
+    loop.run_until_complete(session.close())
+
+
 def test_detach(loop, session) -> None:
     conn = session.connector
     try:
@@ -639,8 +720,43 @@ async def test_ws_connect_unix_socket_allowed_protocols(
 async def test_cookie_jar_usage(loop: Any, aiohttp_client: Any) -> None:
     req_url = None
 
-    jar = mock.Mock()
-    jar.filter_cookies.return_value = None
+    class MockCookieJar(abc.AbstractCookieJar):
+        def __init__(self) -> None:
+            self._update_cookies_mock = mock.Mock()
+            self._filter_cookies_mock = mock.Mock(return_value=BaseCookie())
+            self._clear_mock = mock.Mock()
+            self._clear_domain_mock = mock.Mock()
+            self._items: List[Any] = []
+
+        @property
+        def quote_cookie(self) -> bool:
+            return True
+
+        def clear(self, predicate: Optional[abc.ClearCookiePredicate] = None) -> None:
+            self._clear_mock(predicate)
+
+        def clear_domain(self, domain: str) -> None:
+            self._clear_domain_mock(domain)
+
+        def update_cookies(self, cookies: Any, response_url: URL = URL()) -> None:
+            self._update_cookies_mock(cookies, response_url)
+
+        def filter_cookies(self, request_url: URL) -> BaseCookie[str]:
+            return cast(BaseCookie[str], self._filter_cookies_mock(request_url))
+
+        def __len__(self) -> int:
+            return len(self._items)
+
+        def __iter__(self) -> Iterator[Any]:
+            return iter(self._items)
+
+    jar = MockCookieJar()
+
+    assert jar.quote_cookie is True
+    assert len(jar) == 0
+    assert list(jar) == []
+    jar.clear()
+    jar.clear_domain("example.com")
 
     async def handler(request):
         nonlocal req_url
@@ -657,22 +773,24 @@ async def test_cookie_jar_usage(loop: Any, aiohttp_client: Any) -> None:
     )
 
     # Updating the cookie jar with initial user defined cookies
-    jar.update_cookies.assert_called_with({"request": "req_value"})
+    jar._update_cookies_mock.assert_called_with({"request": "req_value"}, URL())
 
-    jar.update_cookies.reset_mock()
+    jar._update_cookies_mock.reset_mock()
     resp = await session.get("/")
     await resp.release()
 
     # Filtering the cookie jar before sending the request,
     # getting the request URL as only parameter
-    jar.filter_cookies.assert_called_with(URL(req_url))
+    jar._filter_cookies_mock.assert_called_with(URL(req_url))
 
     # Updating the cookie jar with the response cookies
-    assert jar.update_cookies.called
-    resp_cookies = jar.update_cookies.call_args[0][0]
-    assert isinstance(resp_cookies, SimpleCookie)
-    assert "response" in resp_cookies
-    assert resp_cookies["response"].value == "resp_value"
+    assert jar._update_cookies_mock.called
+    resp_cookies = jar._update_cookies_mock.call_args[0][0]
+    # Now update_cookies is called with a list of tuples
+    assert isinstance(resp_cookies, list)
+    assert len(resp_cookies) == 1
+    assert resp_cookies[0][0] == "response"
+    assert resp_cookies[0][1].value == "resp_value"
 
 
 async def test_cookies_with_not_quoted_cookie_jar(
